{*********************************************************}
{*          Web server application components            *}
{*                     version 2.0                       *}
{*         Copyright (c) 2000-2002  S.Perevoznyk         *}
{*                 All rights reserved.                  *}
{*********************************************************}
unit psvBrowser;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  OleCtrls, SHDocVw, ActnList, ActiveX, Menus;


type

  TpsvWebBrowser = class(TWebBrowser)
  private
    FBlankDocumentOnCreate: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    procedure BlankDocument;
    procedure Loaded; override;
    procedure LoadFromString(const HTMLString: string);
    procedure LoadFromStrings(const Strings: TStrings);
    procedure LoadFromStream(const Stream: TStream);
    procedure Print(PromptUser: Boolean);
  published
    property BlankDocumentOnCreate: Boolean read FBlankDocumentOnCreate write FBlankDocumentOnCreate default True;
  end;



function StreamToOleStream(Stream: TStream): IStream;
function StringToOleStream(const AString: string): IStream;

procedure Register;

implementation

uses ComObj;



function StreamToOleStream(Stream: TStream): IStream;
var
  MemHandle: THandle;
begin
  MemHandle := GlobalAlloc(GPTR, Stream.Size);
  if MemHandle <> 0 then begin
    Stream.Read(PChar(MemHandle)^, Stream.Size);
    CreateStreamOnHGlobal(MemHandle, True, Result);
  end else
    Result := nil;
end;

function StringToOleStream(const AString: string): IStream;
var
  MemHandle: THandle;
begin
  MemHandle := GlobalAlloc(GPTR, Length(AString) + 1);
  if MemHandle <> 0 then begin
    Move(AString[1], PChar(MemHandle)^, Length(AString) + 1);
    CreateStreamOnHGlobal(MemHandle, True, Result);
  end else
    Result := nil;
end;

{ TpsvWebBrowser }


procedure TpsvWebBrowser.Print(PromptUser: Boolean);
begin
  if PromptUser then
    ExecWB(OLECMDID_PRINT, OLECMDEXECOPT_PROMPTUSER)
  else
    ExecWB(OLECMDID_PRINT, OLECMDEXECOPT_DONTPROMPTUSER);
end;

procedure TpsvWebBrowser.Loaded;
begin
  inherited;
  if FBlankDocumentOnCreate and not (csDesigning in ComponentState) then
    BlankDocument;
end;


constructor TpsvWebBrowser.Create(AOwner: TComponent);
begin
  inherited;
  FBlankDocumentOnCreate := True;
end;

procedure TpsvWebBrowser.BlankDocument;
var Url : OleVariant;
begin
  Url := 'about:blank';
  Navigate2(Url);
end;

procedure TpsvWebBrowser.LoadFromString(const HTMLString: string);
var
  Stream: IStream;
  StreamInit: IPersistStreamInit;
begin
  Stream := StringToOleStream(HTMLString);
  StreamInit := Document as IPersistStreamInit;
  OleCheck(StreamInit.InitNew);
  OleCheck(StreamInit.Load(Stream));
end;

procedure TpsvWebBrowser.LoadFromStream(const Stream: TStream);
var
  Document: IDispatch;
  OleStream: IStream;
  StreamInit: IPersistStreamInit;
begin
  OleStream := StreamToOleStream(Stream);
  StreamInit := Document as IPersistStreamInit;
  OleCheck(StreamInit.InitNew);
  OleCheck(StreamInit.Load(OleStream));
end;

procedure TpsvWebBrowser.LoadFromStrings(const Strings: TStrings);
begin
  LoadFromString(Strings.Text);
end;

procedure Register;
begin
  RegisterComponents('FastWeb', [TpsvWebBrowser]);
end;

end.
